import copy
import json
import logging
import os
import traceback
from datetime import datetime, timezone
from enum import Enum

import httpx
import opentelemetry.semconv._incubating.attributes.gen_ai_attributes as gen_ai_attributes
import opentelemetry.semconv._incubating.attributes.http_attributes as http_attributes
import opentelemetry.semconv.attributes.server_attributes as server_attributes
from opentelemetry import propagate, trace
from opentelemetry.sdk.trace import SpanProcessor
from opentelemetry.trace import Span, Status, StatusCode, Tracer, set_span_in_context

logger = logging.getLogger(__name__)


OTEL_SERVICE_NAME: str = "mistralai_sdk"
MISTRAL_SDK_OTEL_TRACER_NAME: str = OTEL_SERVICE_NAME + "_tracer"

MISTRAL_SDK_DEBUG_TRACING: bool = os.getenv("MISTRAL_SDK_DEBUG_TRACING", "false").lower() == "true"
DEBUG_HINT: str = "To see detailed tracing logs, set MISTRAL_SDK_DEBUG_TRACING=true."


class MistralAIAttributes:
    MISTRAL_AI_TOTAL_TOKENS = "mistral_ai.request.total_tokens"
    MISTRAL_AI_TOOL_CALL_ARGUMENTS = "mistral_ai.tool.call.arguments"
    MISTRAL_AI_MESSAGE_ID = "mistral_ai.message.id"
    MISTRAL_AI_OPERATION_NAME= "mistral_ai.operation.name"
    MISTRAL_AI_OCR_USAGE_PAGES_PROCESSED = "mistral_ai.ocr.usage.pages_processed"
    MISTRAL_AI_OCR_USAGE_DOC_SIZE_BYTES = "mistral_ai.ocr.usage.doc_size_bytes"
    MISTRAL_AI_OPERATION_ID = "mistral_ai.operation.id"
    MISTRAL_AI_ERROR_TYPE = "mistral_ai.error.type"
    MISTRAL_AI_ERROR_MESSAGE = "mistral_ai.error.message"
    MISTRAL_AI_ERROR_CODE = "mistral_ai.error.code"
    MISTRAL_AI_FUNCTION_CALL_ARGUMENTS = "mistral_ai.function.call.arguments"

class MistralAINameValues(Enum):
    OCR = "ocr"

class TracingErrors(Exception, Enum):
    FAILED_TO_CREATE_SPAN_FOR_REQUEST = "Failed to create span for request."
    FAILED_TO_ENRICH_SPAN_WITH_RESPONSE = "Failed to enrich span with response."
    FAILED_TO_HANDLE_ERROR_IN_SPAN = "Failed to handle error in span."
    FAILED_TO_END_SPAN = "Failed to end span."

    def __str__(self):
        return str(self.value)

class GenAISpanEnum(str, Enum):
    CONVERSATION = "conversation"
    CONV_REQUEST = "POST /v1/conversations"
    EXECUTE_TOOL = "execute_tool"
    VALIDATE_RUN = "validate_run"

    @staticmethod
    def function_call(func_name: str):
        return f"function_call[{func_name}]"


def parse_time_to_nanos(ts: str) -> int:
    dt = datetime.fromisoformat(ts.replace("Z", "+00:00")).astimezone(timezone.utc)
    return int(dt.timestamp() * 1e9)

def set_available_attributes(span: Span, attributes: dict) -> None:
    for attribute, value in attributes.items():
        if value:
            span.set_attribute(attribute, value)


def enrich_span_from_request(span: Span, request: httpx.Request) -> Span:
    if not request.url.port:
        # From httpx doc:
        # Note that the URL class performs port normalization as per the WHATWG spec.
        # Default ports for "http", "https", "ws", "wss", and "ftp" schemes are always treated as None.
        # Handling default ports since most of the time we are using https
        if request.url.scheme == "https":
            port = 443
        elif request.url.scheme == "http":
            port = 80
        else:
            port = -1
    else:
        port = request.url.port

    span.set_attributes({
        http_attributes.HTTP_REQUEST_METHOD: request.method,
        http_attributes.HTTP_URL: str(request.url),
        server_attributes.SERVER_ADDRESS: request.headers.get("host", ""),
        server_attributes.SERVER_PORT: port
    })
    if request._content:
        request_body = json.loads(request._content)

        attributes = {
            gen_ai_attributes.GEN_AI_REQUEST_CHOICE_COUNT: request_body.get("n", None),
            gen_ai_attributes.GEN_AI_REQUEST_ENCODING_FORMATS: request_body.get("encoding_formats", None),
            gen_ai_attributes.GEN_AI_REQUEST_FREQUENCY_PENALTY: request_body.get("frequency_penalty", None),
            gen_ai_attributes.GEN_AI_REQUEST_MAX_TOKENS: request_body.get("max_tokens", None),
            gen_ai_attributes.GEN_AI_REQUEST_MODEL: request_body.get("model", None),
            gen_ai_attributes.GEN_AI_REQUEST_PRESENCE_PENALTY: request_body.get("presence_penalty", None),
            gen_ai_attributes.GEN_AI_REQUEST_SEED: request_body.get("random_seed", None),
            gen_ai_attributes.GEN_AI_REQUEST_STOP_SEQUENCES: request_body.get("stop", None),
            gen_ai_attributes.GEN_AI_REQUEST_TEMPERATURE: request_body.get("temperature", None),
            gen_ai_attributes.GEN_AI_REQUEST_TOP_P: request_body.get("top_p", None),
            gen_ai_attributes.GEN_AI_REQUEST_TOP_K: request_body.get("top_k", None),
            # Input messages are likely to be large, containing user/PII data and other sensitive information.
            # Also structured attributes are not yet supported on spans in Python.
            # For those reasons, we will not record the input messages for now.
            gen_ai_attributes.GEN_AI_INPUT_MESSAGES: None,
        }
        # Set attributes only if they are not None.
        # From OpenTelemetry documentation: None is not a valid attribute value per spec / is not a permitted value type for an attribute.
        set_available_attributes(span, attributes)
    return span


def enrich_span_from_response(tracer: trace.Tracer, span: Span, operation_id: str, response: httpx.Response) -> None:
    span.set_status(Status(StatusCode.OK))
    response_data = json.loads(response.content)

    # Base attributes
    attributes: dict[str, str | int] = {
        http_attributes.HTTP_RESPONSE_STATUS_CODE: response.status_code,
        MistralAIAttributes.MISTRAL_AI_OPERATION_ID: operation_id,
        gen_ai_attributes.GEN_AI_PROVIDER_NAME: gen_ai_attributes.GenAiProviderNameValues.MISTRAL_AI.value
    }

    # Add usage attributes if available
    usage = response_data.get("usage", {})
    if usage:
        attributes.update({
            gen_ai_attributes.GEN_AI_USAGE_PROMPT_TOKENS: usage.get("prompt_tokens", 0),
            gen_ai_attributes.GEN_AI_USAGE_OUTPUT_TOKENS: usage.get("completion_tokens", 0),
            MistralAIAttributes.MISTRAL_AI_TOTAL_TOKENS: usage.get("total_tokens", 0)
        })

    span.set_attributes(attributes)
    if operation_id == "agents_api_v1_agents_create":
        # Semantics from https://opentelemetry.io/docs/specs/semconv/gen-ai/gen-ai-agent-spans/#create-agent-span
        agent_attributes = {
            gen_ai_attributes.GEN_AI_OPERATION_NAME: gen_ai_attributes.GenAiOperationNameValues.CREATE_AGENT.value,
            gen_ai_attributes.GEN_AI_AGENT_DESCRIPTION: response_data.get("description", ""),
            gen_ai_attributes.GEN_AI_AGENT_ID: response_data.get("id", ""),
            gen_ai_attributes.GEN_AI_AGENT_NAME: response_data.get("name", ""),
            gen_ai_attributes.GEN_AI_REQUEST_MODEL: response_data.get("model", ""),
            gen_ai_attributes.GEN_AI_SYSTEM_INSTRUCTIONS: response_data.get("instructions", "")
        }
        span.set_attributes(agent_attributes)
    if operation_id in ["agents_api_v1_conversations_start", "agents_api_v1_conversations_append"]:
        outputs = response_data.get("outputs", [])
        conversation_attributes = {
            gen_ai_attributes.GEN_AI_OPERATION_NAME: gen_ai_attributes.GenAiOperationNameValues.INVOKE_AGENT.value,
            gen_ai_attributes.GEN_AI_CONVERSATION_ID: response_data.get("conversation_id", "")
        }
        span.set_attributes(conversation_attributes)
        parent_context = set_span_in_context(span)

        for output in outputs:
            # TODO: Only enrich the spans if it's a single turn conversation.
            # Multi turn conversations are handled in the extra.run.tools.create_function_result function
            if output["type"] == "function.call":
                pass
            if output["type"] == "tool.execution":
                start_ns = parse_time_to_nanos(output["created_at"])
                end_ns = parse_time_to_nanos(output["completed_at"])
                child_span = tracer.start_span("Tool Execution", start_time=start_ns, context=parent_context)
                child_span.set_attributes({"agent.trace.public": ""})
                tool_attributes = {
                    gen_ai_attributes.GEN_AI_OPERATION_NAME: gen_ai_attributes.GenAiOperationNameValues.EXECUTE_TOOL.value,
                    gen_ai_attributes.GEN_AI_TOOL_CALL_ID: output.get("id", ""),
                    MistralAIAttributes.MISTRAL_AI_TOOL_CALL_ARGUMENTS: output.get("arguments", ""),
                    gen_ai_attributes.GEN_AI_TOOL_NAME: output.get("name", "")
                }
                child_span.set_attributes(tool_attributes)
                child_span.end(end_time=end_ns)
            if output["type"] == "message.output":
                start_ns = parse_time_to_nanos(output["created_at"])
                end_ns = parse_time_to_nanos(output["completed_at"])
                child_span = tracer.start_span("Message Output", start_time=start_ns, context=parent_context)
                child_span.set_attributes({"agent.trace.public": ""})
                message_attributes = {
                    gen_ai_attributes.GEN_AI_OPERATION_NAME: gen_ai_attributes.GenAiOperationNameValues.CHAT.value,
                    gen_ai_attributes.GEN_AI_PROVIDER_NAME: gen_ai_attributes.GenAiProviderNameValues.MISTRAL_AI.value,
                    MistralAIAttributes.MISTRAL_AI_MESSAGE_ID: output.get("id", ""),
                    gen_ai_attributes.GEN_AI_AGENT_ID: output.get("agent_id", ""),
                    gen_ai_attributes.GEN_AI_REQUEST_MODEL: output.get("model", "")
                }
                child_span.set_attributes(message_attributes)
                child_span.end(end_time=end_ns)
    if operation_id == "ocr_v1_ocr_post":
        usage_info = response_data.get("usage_info", "")
        ocr_attributes = {
            MistralAIAttributes.MISTRAL_AI_OPERATION_NAME: MistralAINameValues.OCR.value,
            MistralAIAttributes.MISTRAL_AI_OCR_USAGE_PAGES_PROCESSED: usage_info.get("pages_processed", "") if usage_info else "",
            MistralAIAttributes.MISTRAL_AI_OCR_USAGE_DOC_SIZE_BYTES: usage_info.get("doc_size_bytes", "") if usage_info else "",
            gen_ai_attributes.GEN_AI_REQUEST_MODEL: response_data.get("model", "")
        }
        span.set_attributes(ocr_attributes)


class GenAISpanProcessor(SpanProcessor):
    def on_start(self, span, parent_context = None):
        span.set_attributes({"agent.trace.public": ""})


def get_or_create_otel_tracer() -> tuple[bool, Tracer]:
    """
    Get a tracer from the current TracerProvider.

    The SDK does not set up its own TracerProvider - it relies on the application
    to configure OpenTelemetry. This follows OTEL best practices where:
    - Libraries/SDKs get tracers from the global provider
    - Applications configure the TracerProvider

    If no TracerProvider is configured, the ProxyTracerProvider (default) will
    return a NoOp tracer, effectively disabling tracing. Once the application
    sets up a real TracerProvider, subsequent spans will be recorded.

    Returns:
        Tuple[bool, Tracer]: (tracing_enabled, tracer)
            - tracing_enabled is True if a real TracerProvider is configured
            - tracer is always valid (may be NoOp if no provider configured)
    """
    tracer_provider = trace.get_tracer_provider()
    tracer = tracer_provider.get_tracer(MISTRAL_SDK_OTEL_TRACER_NAME)

    # Tracing is considered enabled if we have a real TracerProvider (not the default proxy)
    tracing_enabled = not isinstance(tracer_provider, trace.ProxyTracerProvider)

    return tracing_enabled, tracer

def get_traced_request_and_span(
    tracing_enabled: bool,
    tracer: Tracer,
    span: Span | None,
    operation_id: str,
    request: httpx.Request,
) -> tuple[httpx.Request, Span | None]:
        if not tracing_enabled:
            return request, span

        try:
            span = tracer.start_span(name=operation_id)
            span.set_attributes({"agent.trace.public": ""})
            # Inject the span context into the request headers to be used by the backend service to continue the trace
            propagate.inject(request.headers, context=set_span_in_context(span))
            span = enrich_span_from_request(span, request)
        except Exception:
            logger.warning(
                "%s %s",
                TracingErrors.FAILED_TO_CREATE_SPAN_FOR_REQUEST,
                traceback.format_exc() if MISTRAL_SDK_DEBUG_TRACING else DEBUG_HINT,
            )
            if span:
                end_span(span=span)
            span = None

        return request, span


def get_traced_response(
    tracing_enabled: bool,
    tracer: Tracer,
    span: Span | None,
    operation_id: str,
    response: httpx.Response,
) -> httpx.Response:
    if not tracing_enabled or not span:
        return response
    try:
        is_stream_response = not response.is_closed and not response.is_stream_consumed
        if is_stream_response:
            return TracedResponse.from_response(resp=response, span=span)
        enrich_span_from_response(
            tracer, span, operation_id, response
        )
    except Exception:
        logger.warning(
            "%s %s",
            TracingErrors.FAILED_TO_ENRICH_SPAN_WITH_RESPONSE,
            traceback.format_exc() if MISTRAL_SDK_DEBUG_TRACING else DEBUG_HINT,
        )
    if span:
        end_span(span=span)
    return response

def get_response_and_error(
    tracing_enabled: bool,
    tracer: Tracer,
    span: Span | None,
    operation_id: str,
    response: httpx.Response,
    error: Exception | None,
) -> tuple[httpx.Response, Exception | None]:
        if not tracing_enabled or not span:
            return response, error
        try:
            if error:
                span.record_exception(error)
                span.set_status(Status(StatusCode.ERROR, str(error)))
            if hasattr(response, "_content") and response._content:
                response_body = json.loads(response._content)
                if response_body.get("object", "") == "error":
                    if error_msg := response_body.get("message", ""):
                        attributes = {
                            http_attributes.HTTP_RESPONSE_STATUS_CODE: response.status_code,
                            MistralAIAttributes.MISTRAL_AI_ERROR_TYPE: response_body.get("type", ""),
                            MistralAIAttributes.MISTRAL_AI_ERROR_MESSAGE: error_msg,
                            MistralAIAttributes.MISTRAL_AI_ERROR_CODE: response_body.get("code", ""),
                        }
                        for attribute, value in attributes.items():
                            if value:
                                span.set_attribute(attribute, value)
            span.end()
            span = None
        except Exception:
            logger.warning(
                "%s %s",
                TracingErrors.FAILED_TO_HANDLE_ERROR_IN_SPAN,
                traceback.format_exc() if MISTRAL_SDK_DEBUG_TRACING else DEBUG_HINT,
            )

            if span:
                span.end()
                span = None
        return response, error


def end_span(span: Span) -> None:
    try:
        span.end()
    except Exception:
        logger.warning(
            "%s %s",
            TracingErrors.FAILED_TO_END_SPAN,
            traceback.format_exc() if MISTRAL_SDK_DEBUG_TRACING else DEBUG_HINT,
        )

class TracedResponse(httpx.Response):
    """
    TracedResponse is a subclass of httpx.Response that ends the span when the response is closed.

    This hack allows ending the span only once the stream is fully consumed.
    """
    def __init__(self, *args, span: Span | None, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.span = span

    def close(self) -> None:
        if self.span:
            end_span(span=self.span)
        super().close()

    async def aclose(self) -> None:
        if self.span:
            end_span(span=self.span)
        await super().aclose()

    @classmethod
    def from_response(cls, resp: httpx.Response, span: Span | None) -> "TracedResponse":
        traced_resp = cls.__new__(cls)
        traced_resp.__dict__ = copy.copy(resp.__dict__)
        traced_resp.span = span

        # Warning: this syntax bypasses the __init__ method.
        # If you add init logic in the TracedResponse.__init__ method, you will need to add the following line for it to execute:
        # traced_resp.__init__(your_arguments)

        return traced_resp



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 1336849c84fb

from __future__ import annotations
from .codeinterpretertool import CodeInterpreterTool, CodeInterpreterToolTypedDict
from .completionargs import CompletionArgs, CompletionArgsTypedDict
from .documentlibrarytool import DocumentLibraryTool, DocumentLibraryToolTypedDict
from .functiontool import FunctionTool, FunctionToolTypedDict
from .imagegenerationtool import ImageGenerationTool, ImageGenerationToolTypedDict
from .websearchpremiumtool import WebSearchPremiumTool, WebSearchPremiumToolTypedDict
from .websearchtool import WebSearchTool, WebSearchToolTypedDict
from datetime import datetime
from functools import partial
from mistralai.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from mistralai.client.utils import validate_const
from mistralai.client.utils.unions import parse_open_union
import pydantic
from pydantic import ConfigDict, model_serializer
from pydantic.functional_validators import AfterValidator, BeforeValidator
from typing import Any, Dict, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


AgentToolTypedDict = TypeAliasType(
    "AgentToolTypedDict",
    Union[
        FunctionToolTypedDict,
        WebSearchToolTypedDict,
        WebSearchPremiumToolTypedDict,
        CodeInterpreterToolTypedDict,
        ImageGenerationToolTypedDict,
        DocumentLibraryToolTypedDict,
    ],
)


class UnknownAgentTool(BaseModel):
    r"""A AgentTool variant the SDK doesn't recognize. Preserves the raw payload."""

    type: Literal["UNKNOWN"] = "UNKNOWN"
    raw: Any
    is_unknown: Literal[True] = True

    model_config = ConfigDict(frozen=True)


_AGENT_TOOL_VARIANTS: dict[str, Any] = {
    "code_interpreter": CodeInterpreterTool,
    "document_library": DocumentLibraryTool,
    "function": FunctionTool,
    "image_generation": ImageGenerationTool,
    "web_search": WebSearchTool,
    "web_search_premium": WebSearchPremiumTool,
}


AgentTool = Annotated[
    Union[
        CodeInterpreterTool,
        DocumentLibraryTool,
        FunctionTool,
        ImageGenerationTool,
        WebSearchTool,
        WebSearchPremiumTool,
        UnknownAgentTool,
    ],
    BeforeValidator(
        partial(
            parse_open_union,
            disc_key="type",
            variants=_AGENT_TOOL_VARIANTS,
            unknown_cls=UnknownAgentTool,
            union_name="AgentTool",
        )
    ),
]


class AgentTypedDict(TypedDict):
    model: str
    name: str
    id: str
    version: int
    versions: List[int]
    created_at: datetime
    updated_at: datetime
    deployment_chat: bool
    source: str
    instructions: NotRequired[Nullable[str]]
    r"""Instruction prompt the model will follow during the conversation."""
    tools: NotRequired[List[AgentToolTypedDict]]
    r"""List of tools which are available to the model during the conversation."""
    completion_args: NotRequired[CompletionArgsTypedDict]
    r"""White-listed arguments from the completion API"""
    description: NotRequired[Nullable[str]]
    handoffs: NotRequired[Nullable[List[str]]]
    metadata: NotRequired[Nullable[Dict[str, Any]]]
    object: Literal["agent"]
    version_message: NotRequired[Nullable[str]]


class Agent(BaseModel):
    model: str

    name: str

    id: str

    version: int

    versions: List[int]

    created_at: datetime

    updated_at: datetime

    deployment_chat: bool

    source: str

    instructions: OptionalNullable[str] = UNSET
    r"""Instruction prompt the model will follow during the conversation."""

    tools: Optional[List[AgentTool]] = None
    r"""List of tools which are available to the model during the conversation."""

    completion_args: Optional[CompletionArgs] = None
    r"""White-listed arguments from the completion API"""

    description: OptionalNullable[str] = UNSET

    handoffs: OptionalNullable[List[str]] = UNSET

    metadata: OptionalNullable[Dict[str, Any]] = UNSET

    object: Annotated[
        Annotated[Optional[Literal["agent"]], AfterValidator(validate_const("agent"))],
        pydantic.Field(alias="object"),
    ] = "agent"

    version_message: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "instructions",
                "tools",
                "completion_args",
                "description",
                "handoffs",
                "metadata",
                "object",
                "version_message",
            ]
        )
        nullable_fields = set(
            ["instructions", "description", "handoffs", "metadata", "version_message"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    Agent.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: feaacfd46dd3

from __future__ import annotations
from .sharingin import SharingIn, SharingInTypedDict
from mistralai.client.types import BaseModel
from mistralai.client.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class LibrariesShareCreateV1RequestTypedDict(TypedDict):
    library_id: str
    sharing_in: SharingInTypedDict


class LibrariesShareCreateV1Request(BaseModel):
    library_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    sharing_in: Annotated[
        SharingIn, FieldMetadata(request=RequestMetadata(media_type="application/json"))
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 26e773725732

from .basesdk import BaseSDK
from mistralai.client import errors, models, utils
from mistralai.client._hooks import HookContext
from mistralai.client.types import OptionalNullable, UNSET
from mistralai.client.utils import get_security_from_env
from mistralai.client.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, Mapping, Optional, Union


class Classifiers(BaseSDK):
    r"""Classifiers API."""

    def moderate(
        self,
        *,
        model: str,
        inputs: Union[
            models.ClassificationRequestInputs,
            models.ClassificationRequestInputsTypedDict,
        ],
        metadata: OptionalNullable[Dict[str, Any]] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ModerationResponse:
        r"""Moderations

        :param model: ID of the model to use.
        :param inputs: Text to classify.
        :param metadata:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ClassificationRequest(
            model=model,
            metadata=metadata,
            inputs=inputs,
        )

        req = self._build_request(
            method="POST",
            path="/v1/moderations",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ClassificationRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="moderations_v1_moderations_post",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ModerationResponse, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def moderate_async(
        self,
        *,
        model: str,
        inputs: Union[
            models.ClassificationRequestInputs,
            models.ClassificationRequestInputsTypedDict,
        ],
        metadata: OptionalNullable[Dict[str, Any]] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ModerationResponse:
        r"""Moderations

        :param model: ID of the model to use.
        :param inputs: Text to classify.
        :param metadata:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ClassificationRequest(
            model=model,
            metadata=metadata,
            inputs=inputs,
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/moderations",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ClassificationRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="moderations_v1_moderations_post",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ModerationResponse, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    def moderate_chat(
        self,
        *,
        inputs: Union[
            models.ChatModerationRequestInputs3,
            models.ChatModerationRequestInputs3TypedDict,
        ],
        model: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ModerationResponse:
        r"""Chat Moderations

        :param inputs: Chat to classify
        :param model:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ChatModerationRequest(
            inputs=utils.get_pydantic_model(
                inputs, models.ChatModerationRequestInputs3
            ),
            model=model,
        )

        req = self._build_request(
            method="POST",
            path="/v1/chat/moderations",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ChatModerationRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="chat_moderations_v1_chat_moderations_post",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ModerationResponse, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def moderate_chat_async(
        self,
        *,
        inputs: Union[
            models.ChatModerationRequestInputs3,
            models.ChatModerationRequestInputs3TypedDict,
        ],
        model: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ModerationResponse:
        r"""Chat Moderations

        :param inputs: Chat to classify
        :param model:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ChatModerationRequest(
            inputs=utils.get_pydantic_model(
                inputs, models.ChatModerationRequestInputs3
            ),
            model=model,
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/chat/moderations",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ChatModerationRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="chat_moderations_v1_chat_moderations_post",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ModerationResponse, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    def classify(
        self,
        *,
        model: str,
        inputs: Union[
            models.ClassificationRequestInputs,
            models.ClassificationRequestInputsTypedDict,
        ],
        metadata: OptionalNullable[Dict[str, Any]] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ClassificationResponse:
        r"""Classifications

        :param model: ID of the model to use.
        :param inputs: Text to classify.
        :param metadata:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ClassificationRequest(
            model=model,
            metadata=metadata,
            inputs=inputs,
        )

        req = self._build_request(
            method="POST",
            path="/v1/classifications",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ClassificationRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="classifications_v1_classifications_post",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ClassificationResponse, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def classify_async(
        self,
        *,
        model: str,
        inputs: Union[
            models.ClassificationRequestInputs,
            models.ClassificationRequestInputsTypedDict,
        ],
        metadata: OptionalNullable[Dict[str, Any]] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ClassificationResponse:
        r"""Classifications

        :param model: ID of the model to use.
        :param inputs: Text to classify.
        :param metadata:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ClassificationRequest(
            model=model,
            metadata=metadata,
            inputs=inputs,
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/classifications",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ClassificationRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="classifications_v1_classifications_post",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ClassificationResponse, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    def classify_chat(
        self,
        *,
        model: str,
        input: Union[models.Inputs, models.InputsTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ClassificationResponse:
        r"""Chat Classifications

        :param model:
        :param input: Chat to classify
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ChatClassificationRequest(
            model=model,
            input=utils.get_pydantic_model(input, models.Inputs),
        )

        req = self._build_request(
            method="POST",
            path="/v1/chat/classifications",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ChatClassificationRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="chat_classifications_v1_chat_classifications_post",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ClassificationResponse, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def classify_chat_async(
        self,
        *,
        model: str,
        input: Union[models.Inputs, models.InputsTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ClassificationResponse:
        r"""Chat Classifications

        :param model:
        :param input: Chat to classify
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ChatClassificationRequest(
            model=model,
            input=utils.get_pydantic_model(input, models.Inputs),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/chat/classifications",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ChatClassificationRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="chat_classifications_v1_chat_classifications_post",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ClassificationResponse, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 01025c12866a

from __future__ import annotations
from .conversationthinkchunk import (
    ConversationThinkChunk,
    ConversationThinkChunkTypedDict,
)
from .documenturlchunk import DocumentURLChunk, DocumentURLChunkTypedDict
from .imageurlchunk import ImageURLChunk, ImageURLChunkTypedDict
from .textchunk import TextChunk, TextChunkTypedDict
from .toolfilechunk import ToolFileChunk, ToolFileChunkTypedDict
from typing import Union
from typing_extensions import TypeAliasType


MessageInputContentChunksTypedDict = TypeAliasType(
    "MessageInputContentChunksTypedDict",
    Union[
        TextChunkTypedDict,
        ImageURLChunkTypedDict,
        DocumentURLChunkTypedDict,
        ConversationThinkChunkTypedDict,
        ToolFileChunkTypedDict,
    ],
)


MessageInputContentChunks = TypeAliasType(
    "MessageInputContentChunks",
    Union[
        TextChunk,
        ImageURLChunk,
        DocumentURLChunk,
        ConversationThinkChunk,
        ToolFileChunk,
    ],
)
